@@ -1,3 +1,5 @@
+from typing import Optional
+
 import chump
 
 from wanikani_notifier.notifiers.notifier import Notifier, NoMessageProvided, factory
@@ -16,11 +18,11 @@ class PushoverNotifier(Notifier):
     def build(cls, app_token: str, user_token: str, **_ignored) -> "PushoverNotifier":
         return PushoverNotifier(app_token, user_token)
 
-    def notify(self, title: str, message: str):
+    def notify(self, title: str, message: str, url: Optional[str] = None, icon: Optional[str] = None):
         if not message:
             raise NoMessageProvided
 
-        self._user.send_message(message, title=title)
+        self._user.send_message(title=title, message=message, url=url)
 
 
 factory.register_builder(PushoverNotifier.key(), PushoverNotifier.build)
